/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-08-27 16:16:29.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.oe.admin.api.controller;

import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.JsonMapper;
import cn.com.qmth.examcloud.core.oe.admin.base.Constants;
import cn.com.qmth.examcloud.core.oe.admin.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.admin.base.utils.excel.ExportUtils;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamRecordDataRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.IllegallyTypeRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.IllegallyTypeEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.enums.AuditStatus;
import cn.com.qmth.examcloud.core.oe.admin.dao.enums.SelectType;
import cn.com.qmth.examcloud.core.oe.admin.service.AsyncExportService;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamAuditService;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examaudit.*;
import cn.com.qmth.examcloud.core.oe.admin.service.util.AsyncExportConcurrentUtil;
import cn.com.qmth.examcloud.reports.commons.bean.AdminOperateReport;
import cn.com.qmth.examcloud.reports.commons.util.ReportsUtil;
import cn.com.qmth.examcloud.support.enums.ExamProperties;
import cn.com.qmth.examcloud.support.helper.ExamCacheTransferHelper;
import cn.com.qmth.examcloud.support.helper.IdentityNumberHelper;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import cn.com.qmth.examcloud.web.support.Naked;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * 考试记录审核相关接口
 *
 * @author: QMTH
 * @since: 2018/8/27
 */
@RestController
@Api(tags = "考试记录审核相关接口")
@RequestMapping("${$rmp.ctr.oe}/exam/audit")
public class ExamAuditController extends ControllerSupport {

    @Autowired
    private ExamAuditService examAuditService;

    @Autowired
    private ExamRecordDataRepo examRecordDataRepo;

    @Autowired
    private IllegallyTypeRepo illegallyTypeRepo;

    @Autowired
    private AsyncExportService asyncExportService;

    @PostMapping("/list")
    @ApiOperation(value = "查询“监考已审”列表(分页)")
    public Page<ExamAuditInfo> getExamAuditList(@RequestBody ExamAuditQuery query) {
        User user = getAccessUser();
        query.setRootOrgId(user.getRootOrgId());
        Page<ExamAuditInfo> examAuditList = examAuditService.getExamAuditList(query);

        examAuditList.getContent().forEach(p -> {
            p.setIdentityNumber(IdentityNumberHelper.conceal(user.getRootOrgId(), p.getIdentityNumber()));
        });
        return examAuditList;
    }

    @GetMapping("export/async")
    @ApiOperation(value = "导出“监考已审”(异步)")
    public void exportExamAuditList(@RequestParam String query) {
        User user = getAccessUser();
        asyncExportService.exportExamAuditList(query, user);
        AsyncExportConcurrentUtil.checkOrStartJob();
        ReportsUtil.report(new AdminOperateReport(user.getRootOrgId(), user.getUserId(), "监考已审-导出", "导出条件:" + query));
    }

    @PostMapping("/discipline/list")
    @ApiOperation(value = "查询“违纪名单”列表(分页)")
    public Page<ExamAuditInfo> getExamAuditUnPassList(@RequestBody ExamAuditQuery query) {
        query.setStatus(AuditStatus.UN_PASS.name());
        User user = getAccessUser();
        query.setRootOrgId(user.getRootOrgId());
        Page<ExamAuditInfo> examAuditList = examAuditService.getExamAuditList(query);

        examAuditList.getContent().forEach(p -> {
            p.setIdentityNumber(IdentityNumberHelper.conceal(user.getRootOrgId(), p.getIdentityNumber()));
        });
        return examAuditList;
    }

    @Naked
    @GetMapping("/discipline/list/export")
    @ApiOperation(value = "导出“违纪名单”列表(Excel)", notes = "参数示例：query={\"pageNo\":1,\"pageSize\":10,\"examId\":123, ...}")
    public void exportExamAuditUnPassList(@RequestParam String query, HttpServletResponse response) throws Exception {
        ExamAuditQuery newQuery = new JsonMapper().parseJson(query, ExamAuditQuery.class);
        Check.isNull(newQuery, "请求参数不能为空！");
        Check.isNull(newQuery.getExamId(), "请先选择考试批次！");

        newQuery.setStatus(AuditStatus.UN_PASS.name());
        newQuery.setSelectType(SelectType.EXPORT);
        Page<ExamAuditInfo> page = examAuditService.getExamAuditList(newQuery);
        List<ExamAuditExcel> list = ExamAuditEntityConvert.ofExcel(page);
        ExportUtils.exportEXCEL("违纪名单列表", ExamAuditExcel.class, list, response);
    }

    @PostMapping(value = "/single/audit")
    @ApiOperation(value = "监考待审-单个审核")
    public void singleAudit(@RequestParam(required = true) Long examRecordDataId,
                            @RequestParam(required = false) String disciplineDetail,
                            @RequestParam(required = true) Boolean isPass,
                            @RequestParam(required = false) String disciplineType,
                            @RequestParam(required = false) Long illegallyTypeId) {
        if (examRecordDataId == null) {
            throw new StatusException("singleAudit-001", "examRecordDataId不能为空");
        }
        if (isPass == null) {
            throw new StatusException("singleAudit-002", "isPass不能为空");
        }
        User user = getAccessUser();
        examAuditService.singleAudit(examRecordDataId, isPass, disciplineDetail, disciplineType, user, illegallyTypeId);
        StringBuilder sb = new StringBuilder();
        sb.append("考试记录ID:" + examRecordDataId);
        if (isPass) {
            sb.append(" 审核状态:通过");
        } else {
            sb.append(" 审核状态:不通过");
        }
        if (illegallyTypeId != null) {
            IllegallyTypeEntity illegallyType =
                    GlobalHelper.getEntity(illegallyTypeRepo, illegallyTypeId, IllegallyTypeEntity.class);
            sb.append(" 违纪类型:" + illegallyType.getName());
        }
        if (StringUtils.isNotEmpty(disciplineDetail)) {
            sb.append(" 详情描述:" + disciplineDetail);
        }
        ReportsUtil.report(new AdminOperateReport(user.getRootOrgId(), user.getUserId(), "监考待审-审核", sb.toString()));
    }

    @PostMapping(value = "/batch/audit")
    @ApiOperation(value = "监考待审-批量审核")
    public void batchAudit(@RequestParam(required = true) List<Long> examRecordDataIds,
                           @RequestParam(required = false) String disciplineDetail,
                           @RequestParam(required = true) Boolean isPass,
                           @RequestParam(required = false) String disciplineType,
                           @RequestParam(required = false) Long illegallyTypeId) {
        if (examRecordDataIds == null || examRecordDataIds.size() == 0) {
            throw new StatusException("batchAudit-001", "examRecordDataIds不能为空");
        }
        if (isPass == null) {
            throw new StatusException("batchAudit-002", "isPass不能为空");
        }
        User user = getAccessUser();
        for (Long examRecordDataId : examRecordDataIds) {
            examAuditService.singleAudit(examRecordDataId, isPass, disciplineDetail, disciplineType, user, illegallyTypeId);
        }
        StringBuilder sb = new StringBuilder();
        if (isPass) {
            sb.append("审核状态:通过");
        } else {
            sb.append("审核状态:不通过");
        }
        sb.append(" 审核条数:" + examRecordDataIds.size());
        ReportsUtil.report(new AdminOperateReport(user.getRootOrgId(), user.getUserId(), "监考待审-批量审核", sb.toString()));
    }

    @PostMapping(value = "/redoAudit")
    @ApiOperation(value = "重新审核")
    public void redoAudit(@RequestBody RedoAuditInfo redoAuditInfo) {
        User user = getAccessUser();
        if (redoAuditInfo.getExamRecordDataIds() == null || redoAuditInfo.getExamRecordDataIds().size() == 0) {
            throw new StatusException("redoAudit-001", "examRecordDataIds不能为空");
        }
        if (redoAuditInfo.getIsPass() == null) {
            throw new StatusException("redoAudit-002", "isPass不能为空");
        }
        if (!redoAuditInfo.getIsPass() && null == redoAuditInfo.getIllegallyTypeId()) {
            throw new StatusException("redoAudit-003", "审核为不通过时违纪类型不能为空");
        }
        Set<Long> examIds = new HashSet<Long>();
        for (Long examRecordDataId : redoAuditInfo.getExamRecordDataIds()) {
            ExamRecordDataEntity examRecordData = GlobalHelper.getEntity(examRecordDataRepo, examRecordDataId, ExamRecordDataEntity.class);
            examIds.add(examRecordData.getExamId().longValue());
        }
        if (examIds.size() > 1) {
            throw new StatusException("redoAudit-004", "考试ID中包含了多个批次");
        }
        long examId = examIds.iterator().next();

        //原需求：已经生成了评卷任务，不能重新审为不通过，但是可以审核为已通过
        //20191220,需求临时调整：已经生成了评卷任务,均不可重审
        //20200923,需求调整，恢复至原需求:已经生成了评卷任务，不能重新审为不通过，但是可以审核为已通过
        String markingTaskBuilt =
                ExamCacheTransferHelper.getDefaultCachedExamProperty(examId, ExamProperties.MARKING_TASK_BUILDED.name()).getValue();
        if (StringUtils.isNotBlank(markingTaskBuilt) && Constants.isTrue.equals(markingTaskBuilt)) {
            if (!redoAuditInfo.getIsPass()) {
                throw new StatusException("redoAudit-005", "该考试评卷任务已生成,不允许重审");
            }
        }
        examAuditService.redoAudit(redoAuditInfo, user);
        StringBuilder sb = new StringBuilder();
        sb.append("考试记录ID:" + redoAuditInfo.getExamRecordDataIds().get(0));
        if (redoAuditInfo.getIsPass()) {
            sb.append(" 审核状态:通过");
        } else {
            sb.append(" 审核状态:不通过");
        }
        if (redoAuditInfo.getIllegallyTypeId() != null) {
            IllegallyTypeEntity illegallyType =
                    GlobalHelper.getEntity(illegallyTypeRepo, redoAuditInfo.getIllegallyTypeId(), IllegallyTypeEntity.class);
            sb.append(" 违纪类型:" + illegallyType.getName());
        }
        if (StringUtils.isNotEmpty(redoAuditInfo.getDisciplineDetail())) {
            sb.append(" 详情描述:" + redoAuditInfo.getDisciplineDetail());
        }
        ReportsUtil.report(new AdminOperateReport(user.getRootOrgId(), user.getUserId(), "考试明细-重审", sb.toString()));
    }

}
