/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-08-27 14:52:17.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.oe.admin.api.controller;

import cn.com.qmth.examcloud.api.commons.enums.ExamType;
import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.util.JsonMapper;
import cn.com.qmth.examcloud.core.oe.admin.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.admin.base.utils.excel.ExportUtils;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamRecordDataRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamRecordFileAnswerRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamRecordFileAnswerEntity;
import cn.com.qmth.examcloud.core.oe.admin.service.AsyncExportService;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamCaptureService;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamRecordService;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examrecord.ExamRecordFileAnswerInfo;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examrecord.ExamRecordInfo;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examrecord.ExamRecordQuery;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examrecord.ExamStudentQuestionScoreInfo;
import cn.com.qmth.examcloud.core.oe.admin.service.util.AsyncExportConcurrentUtil;
import cn.com.qmth.examcloud.reports.commons.bean.AdminOperateReport;
import cn.com.qmth.examcloud.reports.commons.util.ReportsUtil;
import cn.com.qmth.examcloud.support.filestorage.FileStorageUtil;
import cn.com.qmth.examcloud.support.helper.IdentityNumberHelper;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import cn.com.qmth.examcloud.web.support.Naked;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;

/**
 * 考试记录相关接口
 *
 * @author: QMTH
 * @since: 2018/8/27
 */
@RestController
@Api(tags = "考试记录相关接口")
@RequestMapping("${$rmp.ctr.oe}/exam/record")
public class ExamRecordController extends ControllerSupport {

    @Autowired
    private ExamRecordService examRecordService;

    @Autowired
    private AsyncExportService asyncExportService;

    @Autowired
    private ExamRecordDataRepo examRecordDataRepo;

    @Autowired
    private ExamCaptureService examCaptureService;

    @Autowired
    private ExamRecordFileAnswerRepo examRecordFileAnswerRepo;


    @PostMapping("/waiting/audit/list")
    @ApiOperation(value = "查询“监考待审”列表(分页)")
    public Page<ExamRecordInfo> getExamRecordWaitingAuditList(@RequestBody ExamRecordQuery query) {
        Page<ExamRecordInfo> examRecordWaitingAuditList = examRecordService.getExamRecordWaitingAuditList(query);
        examRecordWaitingAuditList.getContent().forEach(p -> {
            p.setIdentityNumber(IdentityNumberHelper.conceal(p.getRootOrgId(), p.getIdentityNumber()));
        });
        List<ExamRecordInfo> examRecordInfoList = examRecordWaitingAuditList.getContent();
        if (examRecordInfoList != null && examRecordInfoList.size() > 0) {
            String examType = examRecordInfoList.get(0).getExamType();

            for (ExamRecordInfo examRecordInfo : examRecordInfoList) {
                if (ExamType.ONLINE.name().equals(examType) || ExamType.ONLINE_HOMEWORK.name().equals(examType)) {
                    examRecordInfo.setVirtualCameraNames(examCaptureService.getVirtualCameraNames(examRecordInfo.getDataId()));
                    examRecordInfo.setHasVirtual(StringUtils.isNotBlank(examRecordInfo.getVirtualCameraNames()));
                }
            }
        }
        return examRecordWaitingAuditList;
    }

    @PostMapping("/waiting/audit/next")
    @ApiOperation(value = "查询“监考待审”下一条记录")
    public Long getExamRecordWaitingAuditNextId(@RequestBody ExamRecordQuery query,
                                                @RequestParam Long examRecordDataId, String next) {
        return examRecordService.getExamRecordWaitingAuditNextId(query, examRecordDataId, next);
    }

    @PostMapping("/detail/check")
    @ApiOperation(value = "查询“考试明细” 是否存在待审数据")
    public Long existsWarnExamRecordDetail(@RequestBody ExamRecordQuery query) {
        return examRecordService.existsWarnExamRecordDetail(query);
    }

    @PostMapping("/detail/list")
    @ApiOperation(value = "查询“考试明细”列表(分页)")
    public Page<ExamRecordInfo> getExamRecordDetailList(@RequestBody ExamRecordQuery query) {
        Page<ExamRecordInfo> examRecordInfoPage = examRecordService.getExamRecordDetailListForPage(query);
        List<ExamRecordInfo> examRecordInfoList = examRecordInfoPage.getContent();
        if (examRecordInfoList != null && examRecordInfoList.size() > 0) {
            String examType = examRecordInfoList.get(0).getExamType();

            for (ExamRecordInfo examRecordInfo : examRecordInfoList) {
                examRecordInfo.setIdentityNumber((IdentityNumberHelper.conceal(examRecordInfo.getRootOrgId(),
                        examRecordInfo.getIdentityNumber())));

                if (ExamType.OFFLINE.name().equals(examType)) {
                    long examRecordDataId = examRecordInfo.getDataId();
                    List<ExamRecordFileAnswerEntity> fileAnswerList =
                            examRecordFileAnswerRepo.findByExamRecordDataId(examRecordDataId);
                    if (fileAnswerList != null) {
                        examRecordInfo.setOfflineFiles(getOfflineFilesFrom(fileAnswerList));
                    }
                }
                if (ExamType.ONLINE.name().equals(examType) || ExamType.ONLINE_HOMEWORK.name().equals(examType)) {
                    examRecordInfo.setVirtualCameraNames(examCaptureService.getVirtualCameraNames(examRecordInfo.getDataId()));
                }
            }
        }
        return examRecordInfoPage;
    }

    private List<ExamRecordFileAnswerInfo> getOfflineFilesFrom(List<ExamRecordFileAnswerEntity> fileAnswerList) {
        List<ExamRecordFileAnswerInfo> resultList = new ArrayList<>();
        for (ExamRecordFileAnswerEntity entity : fileAnswerList) {
            ExamRecordFileAnswerInfo info = new ExamRecordFileAnswerInfo();
            info.setId(entity.getId());
            info.setExamRecordDataId(entity.getExamRecordDataId());
            info.setOfflineFileUrl(FileStorageUtil.realPath(entity.getFileUrl()));
            info.setOfflineFileName(entity.getFileName());
            info.setOriginalFileName(entity.getOriginalFileName());
            info.setFileType(entity.getFileType());
            info.setSuffix(entity.getSuffix());
            info.setProperties(entity.getProperties());
            resultList.add(info);
        }

        return resultList;
    }

    @Naked
    @GetMapping("/detail/list/export")
    @ApiOperation(value = "导出“考试明细”列表(Excel)", notes = "参数示例：query={\"pageNo\":1,\"pageSize\":10,\"examId\":123, ...}")
    public void exportExamRecordDetailList(@RequestParam String query, HttpServletResponse response) throws Exception {
        ExamRecordQuery newQuery = new JsonMapper().parseJson(query, ExamRecordQuery.class);
        Check.isNull(newQuery, "请求参数不能为空！");
        Check.isNull(newQuery.getExamId(), "请先选择考试批次！");

        List<ExamRecordInfo> examRecordInfoList = examRecordService.getExamRecordDetailList(newQuery);
        if (examRecordInfoList != null && examRecordInfoList.size() > 0) {
            String examType = examRecordInfoList.get(0).getExamType();

            for (ExamRecordInfo examRecordInfo : examRecordInfoList) {
                if (ExamType.ONLINE.name().equals(examType) || ExamType.ONLINE_HOMEWORK.name().equals(examType)) {
                    examRecordInfo.setVirtualCameraNames(examCaptureService.getVirtualCameraNames(examRecordInfo.getDataId()));
                }
            }
        }

        ExportUtils.exportEXCEL("考试明细列表", ExamRecordInfo.class, examRecordInfoList, response);
    }

    @GetMapping("/detail/list/export/async")
    @ApiOperation(value = "导出“考试明细”列表(异步)", notes = "参数示例：query={\"rootOrgId\":0,\"examId\":1}")
    public void exportExamRecordDetails(@RequestParam String query) {
        User user = getAccessUser();
        asyncExportService.exportExamRecordDetails(query, user);
        AsyncExportConcurrentUtil.checkOrStartJob();
        ReportsUtil.report(new AdminOperateReport(user.getRootOrgId(), user.getUserId(), "考试明细-导出", "导出条件:" + query));
    }

    @PostMapping(value = "/refresh/capture/statistic")
    @ApiOperation(value = "监考待审-重新统计", notes = "根据人脸识别阀值重新计算数据")
    public ResponseEntity<String> refreshCaptureStatistic(@RequestParam Long examId) {
        examRecordService.refreshCaptureStatistic(examId);
        return new ResponseEntity<>(HttpStatus.OK);
    }

    @PostMapping(value = "/clean/capture/statistic")
    @ApiOperation(value = "监考待审-清理数据")
    public ResponseEntity<String> cleanCaptureStatistic() {
        //todo
        //examRecordService.cleanCaptureStatistic();
        return new ResponseEntity<>(HttpStatus.OK);
    }

    @GetMapping("/select/byExamStudentId")
    @ApiOperation(value = "根据考生ID查询考试记录")
    public List<ExamRecordDataEntity> getExamRecordByExamStudentId(@RequestParam Long examStudentId) {
        List<ExamRecordDataEntity> examRecordEntityList = examRecordDataRepo.findByExamStudentId(examStudentId);
        return examRecordEntityList;
    }

    @GetMapping("/detail/examStudentQuestionScoreList")
    @ApiOperation(value = "导出考生作答分数明细列表(Excel)", notes = "参数examId和examId必传")
    public void getExamStudentQuestionScoreList(@RequestParam Long examId, @RequestParam String courseCode,
                                                HttpServletResponse response) throws Exception {
        Check.isNull(examId, "考试id不能为空！");
        Check.isBlank(courseCode, "课程代码不能空！");
        List<ExamStudentQuestionScoreInfo> examRecordInfoList =
                examRecordService.getExamStudentQuestionScoreList(examId, courseCode);
        ExportUtils.exportEXCEL("考试明细列表", ExamStudentQuestionScoreInfo.class, examRecordInfoList, response);
    }

}
