package cn.com.qmth.examcloud.core.oe.admin.api.provider;

import cn.com.qmth.examcloud.api.commons.enums.ExamType;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.basic.api.CourseCloudService;
import cn.com.qmth.examcloud.core.basic.api.bean.CourseBean;
import cn.com.qmth.examcloud.core.basic.api.request.GetCourseReq;
import cn.com.qmth.examcloud.core.basic.api.response.GetCourseResp;
import cn.com.qmth.examcloud.core.oe.admin.api.ExamScoreDataCloudService;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.ExamScoreDataBean;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.QueryCapturePhotoBean;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.ScoreDataBean;
import cn.com.qmth.examcloud.core.oe.admin.api.request.*;
import cn.com.qmth.examcloud.core.oe.admin.api.response.*;
import cn.com.qmth.examcloud.core.oe.admin.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.admin.dao.*;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.*;
import cn.com.qmth.examcloud.core.oe.admin.dao.enums.ExamRecordStatus;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamScoreService;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamStudentFinalScoreService;
import cn.com.qmth.examcloud.core.oe.admin.service.GainBaseDataService;
import cn.com.qmth.examcloud.core.oe.admin.service.IllegallyTypeService;
import cn.com.qmth.examcloud.examwork.api.ExamCloudService;
import cn.com.qmth.examcloud.examwork.api.bean.ExamBean;
import cn.com.qmth.examcloud.examwork.api.request.GetExamReq;
import cn.com.qmth.examcloud.examwork.api.response.GetExamResp;
import cn.com.qmth.examcloud.support.Constants;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.support.cache.bean.CourseCacheBean;
import cn.com.qmth.examcloud.support.cache.bean.ExamSettingsCacheBean;
import cn.com.qmth.examcloud.support.cache.bean.SysPropertyCacheBean;
import cn.com.qmth.examcloud.support.helper.ExamCacheTransferHelper;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.json.JSONException;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.persistence.criteria.Predicate;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

@Api(tags = "考试分数信息相关接口")
@RestController
@RequestMapping("${$rmp.cloud.oe}/examScoreData")
public class ExamScoreDataCloudServiceProvider extends ControllerSupport implements ExamScoreDataCloudService {

    private static final long serialVersionUID = 8280332746440316604L;

    @Autowired
    private ExamScoreRepo examScoreRepo;

    @Autowired
    private ExamRecordDataRepo examRecordDataRepo;

    @Autowired
    private ExamStudentRepo examStudentRepo;

    @Autowired
    private ExamAuditRepo examAuditRepo;

    @Autowired
    private ExamCaptureRepo examCaptureRepo;

    @Autowired
    private GainBaseDataService gainBaseDataService;

    @Autowired
    private ExamCloudService examCloudService;

    @Autowired
    private CourseCloudService courseCloudService;

    @Autowired
    private ExamScoreService examScoreService;

    @Autowired
    private ExamStudentFinalScoreService examStudentFinalScoreService;

    @Autowired
    IllegallyTypeService illegallyTypeService;

    @Override
    @ApiOperation(value = "根据分数id查询考试记录，学生信息，分数信息等数据")
    @PostMapping("/findExamScoreDataByScoreId")
    public FindExamScoreDataResp findExamScoreDataByScoreId(@RequestBody FindExamScoreDataReq findExamScoreDataReq) {
        long scoreId = findExamScoreDataReq.getScoreId();
        ExamScoreEntity examScore = GlobalHelper.getEntity(examScoreRepo, scoreId, ExamScoreEntity.class);
        ExamRecordDataEntity examRecordDataEntity = GlobalHelper.getEntity(examRecordDataRepo, examScore.getExamRecordDataId(), ExamRecordDataEntity.class);
        ExamStudentEntity examStudentEntity = examStudentRepo.findByExamStudentId(examRecordDataEntity.getExamStudentId());

        FindExamScoreDataResp findExamScoreDataResp = new FindExamScoreDataResp();
        List<ExamScoreDataBean> examScoreDatas = new ArrayList<ExamScoreDataBean>();
        examScoreDatas.add(buildExamScoreDataBean(examScore, examRecordDataEntity, examStudentEntity));
        findExamScoreDataResp.setExamScoreDatas(examScoreDatas);

        return findExamScoreDataResp;
    }

    @Override
    @ApiOperation(value = "根据examIdAndStudentCode查询考试记录，学生信息，分数信息等数据")
    @PostMapping("/findExamScoreDataByExamIdAndStudentCode")
    public FindExamScoreDataResp findExamScoreDataByExamIdAndStudentCode(@RequestBody FindExamScoreDataReq findExamScoreDataReq) {
        Long examId = findExamScoreDataReq.getExamId();
        String studentCode = findExamScoreDataReq.getStudentCode();
        List<ExamRecordDataEntity> examRecordDataList = examRecordDataRepo.findByExamIdAndStudentCode(examId, studentCode);

        List<ExamScoreDataBean> examScoreDatas = new ArrayList<ExamScoreDataBean>();
        for (ExamRecordDataEntity examRecordData : examRecordDataList) {
            ExamScoreEntity examScore = examScoreRepo.findByExamRecordDataId(examRecordData.getId());
            ExamStudentEntity examStudent = examStudentRepo.findByExamStudentId(examRecordData.getExamStudentId());
            examScoreDatas.add(buildExamScoreDataBean(examScore, examRecordData, examStudent));
        }

        FindExamScoreDataResp findExamScoreDataResp = new FindExamScoreDataResp();
        findExamScoreDataResp.setExamScoreDatas(examScoreDatas);
        return findExamScoreDataResp;
    }

    private ExamScoreDataBean buildExamScoreDataBean(
            ExamScoreEntity examScore, ExamRecordDataEntity examRecordDataEntity, ExamStudentEntity examStudentEntity) {
        ExamScoreDataBean examScoreDataBean = new ExamScoreDataBean();
        examScoreDataBean.setExamId(examStudentEntity.getExamId());
        examScoreDataBean.setStudentName(examStudentEntity.getStudentName());
        examScoreDataBean.setStudentCode(examStudentEntity.getStudentCode());
        examScoreDataBean.setIdentityNumber(examStudentEntity.getIdentityNumber());

        CourseCacheBean courseBean = ExamCacheTransferHelper.getCachedCourse(examStudentEntity.getCourseId());
        examScoreDataBean.setCourseName(courseBean.getName());
        examScoreDataBean.setCourseCode(examStudentEntity.getCourseCode());
        examScoreDataBean.setStartTime(examRecordDataEntity.getStartTime());
        examScoreDataBean.setEndTime(examRecordDataEntity.getEndTime());
        examScoreDataBean.setSuccPercent(examRecordDataEntity.getFaceSuccessPercent());
        examScoreDataBean.setIsIllegality(examRecordDataEntity.getIsIllegality());
        examScoreDataBean.setScoreId(examScore.getId());
        examScoreDataBean.setTotalScore(examScore.getTotalScore());
        examScoreDataBean.setObjectiveScore(examScore.getObjectiveScore());
        examScoreDataBean.setSubjectiveScore(examScore.getSubjectiveScore());
        return examScoreDataBean;
    }

    @Override
    @ApiOperation(value = "根据examRecordDataId查询成绩数据")
    @PostMapping("/getScoreData")
    public GetScoreDataResp getScoreData(@RequestBody GetScoreDataReq req) {
        Long examRecordDataId = req.getExamRecordDataId();
        ExamRecordDataEntity examRecordData = GlobalHelper.getEntity(examRecordDataRepo, examRecordDataId, ExamRecordDataEntity.class);
        if (null == examRecordData) {
            throw new StatusException("getScoreData-exception", "请求参数不正确");
        }
        ExamScoreEntity examScore = examScoreRepo.findByExamRecordDataId(examRecordDataId);

        GetScoreDataResp resp = new GetScoreDataResp();
        resp.setExamRecordDataId(examRecordDataId);
        resp.setStudentCode(examRecordData.getStudentCode());
        resp.setStudentName(examRecordData.getStudentName());
        resp.setIdentityNumber(examRecordData.getIdentityNumber());

        CourseCacheBean courseBean = ExamCacheTransferHelper.getCachedCourse(examRecordData.getCourseId());
        resp.setCourseCode(courseBean.getCode());
        resp.setCourseName(courseBean.getName());
        resp.setStartTime(examRecordData.getStartTime());
        resp.setEndTime(examRecordData.getEndTime());
        resp.setTotalScore(examScore.getTotalScore());

        resp.setIsWarn(examRecordData.getIsWarn());
        resp.setIsAudit(examRecordData.getIsAudit());
        resp.setIsIllegality(examRecordData.getIsIllegality());

        return resp;
    }

    @Override
    @ApiOperation(value = "根据examRecordDataId查询成绩审核数据")
    @PostMapping("/getAuditData")
    public GetAuditDataResp getAuditData(@RequestBody GetAuditDataReq req) {
        Check.isEmpty(req.getExamRecordDataId(), "考试记录ID不能为空");
        ExamAuditEntity examAuditEntity = examAuditRepo.findByExamRecordDataId(req.getExamRecordDataId());
        //考试记录
        ExamRecordDataEntity examRecordData = GlobalHelper.getEntity(examRecordDataRepo, req.getExamRecordDataId(), ExamRecordDataEntity.class);
        if (null == examRecordData) {
            throw new StatusException("100001", "找不到考试记录id为" + req.getExamRecordDataId() + "数据");
        }
        GetAuditDataResp resp = new GetAuditDataResp();
        if (examAuditEntity != null) {
            resp.setExamRecordDataId(examAuditEntity.getExamRecordDataId());
            String disciplineType = examAuditEntity.getDisciplineType();
            if (disciplineType != null) {
                IllegallyTypeEntity illegallyType =
                        illegallyTypeService.getIllegallyType(examRecordData.getRootOrgId(), disciplineType);

                resp.setDisciplineType(illegallyType.getName());
            }
            resp.setDisciplineDetail(examAuditEntity.getDisciplineDetail());
            resp.setCreationTime(examAuditEntity.getCreationTime());
            resp.setAuditUserName(examAuditEntity.getAuditUserName());
            //是否违纪
            resp.setIsDiscipline(examRecordData.getIsIllegality() ? 1 : 0);
        }
        return resp;
    }

    @Override
    @ApiOperation(value = "根据examRecordDataId查询抓拍照片数据")
    @PostMapping("/queryCapturePhoto")
    public QueryCapturePhotoResp queryCapturePhoto(@RequestBody QueryCapturePhotoReq req) {
        Check.isEmpty(req.getExamRecordDataId(), "考试记录ID不能为空");
        List<ExamCaptureEntity> examCaptureList = examCaptureRepo.findByExamRecordDataId(req.getExamRecordDataId());
        QueryCapturePhotoResp queryCapturePhotoResp = new QueryCapturePhotoResp();
        queryCapturePhotoResp.setExamRecordDataId(req.getExamRecordDataId());
        List<QueryCapturePhotoBean> capturePhotoList = new ArrayList<QueryCapturePhotoBean>();
        if (examCaptureList.size() > 0) {
            for (ExamCaptureEntity examCaptureEntity : examCaptureList) {
                QueryCapturePhotoBean queryCapturePhotoBean = new QueryCapturePhotoBean();
                queryCapturePhotoBean.setFileUrl(examCaptureEntity.getFileUrl());
                queryCapturePhotoBean.setIsPass(examCaptureEntity.getIsPass());
                queryCapturePhotoBean.setIsStranger(examCaptureEntity.getIsStranger());
                String faceLivenessResult = examCaptureEntity.getFacelivenessResult();
                queryCapturePhotoBean.setIsLivenessPass(isLivenessPass(faceLivenessResult));
                capturePhotoList.add(queryCapturePhotoBean);
            }
        }
        queryCapturePhotoResp.setCapturePhotoList(capturePhotoList);
        return queryCapturePhotoResp;
    }

    /**
     * 人脸真实性是否通过
     *
     * @param faceLivenessResult
     * @return
     */
    private Boolean isLivenessPass(String faceLivenessResult) {
        if (StringUtils.isNotBlank(faceLivenessResult)) {
            JSONObject jsonObject;
            try {
                jsonObject = new JSONObject(faceLivenessResult);
                if (jsonObject.has("error_code") && jsonObject.getInt("error_code") == 0 && jsonObject.has("result")) {
                    JSONObject resultJson = jsonObject.getJSONObject("result");
                    if (resultJson.has("face_liveness")) {
                        double faceLivenessVal = resultJson.getDouble("face_liveness");

                        Double baiduFacelivenessThreshold;
                        SysPropertyCacheBean baiduFacelivenessThresholdProperty = CacheHelper.getSysProperty("$baidu.faceliveness.threshold");
                        if (!baiduFacelivenessThresholdProperty.getHasValue()) {
                            baiduFacelivenessThreshold = Constants.DEFAULT_BAIDU_FACELIVENESS_THRESHOLD;
                        } else {
                            baiduFacelivenessThreshold = Double.valueOf(baiduFacelivenessThresholdProperty.getValue().toString());
                        }

                        if (faceLivenessVal > baiduFacelivenessThreshold) {
                            return true;
                        }
                    }
                }
            } catch (JSONException e) {
                e.printStackTrace();
                return false;
            }

        }
        return false;
    }

    @Override
    @ApiOperation(value = "按考试名称，rootOrgId,courseCode，身份证号或学号查询成绩信息")
    @PostMapping("/queryScoreData")
    public QueryScoreDataResp queryScoreData(@RequestBody QueryScoreDataReq req) {
        /*if (StringUtils.isBlank(req.getExamName())) {
            throw new StatusException("OE-ADMIN-1000005", "考试名称不能为空");
        }*/
        if (req.getRootOrgId() == null) {
            throw new StatusException("OE-ADMIN-1000005", "rootOrgId不能为空");
        }
        if (StringUtils.isBlank(req.getCourseCode())) {
            throw new StatusException("OE-ADMIN-1000005", "课程code不能为空");
        }
        if (StringUtils.isBlank(req.getIdentityNumber()) && StringUtils.isBlank(req.getStudentCode())) {
            throw new StatusException("OE-ADMIN-1000005", "身份证号和学号不能都为空");
        }

        GetExamReq examReq = new GetExamReq();
        examReq.setRootOrgId(req.getRootOrgId());
        //如果考试代码为空，则将考试名称作为考试代码
        if (StringUtils.isEmpty(req.getExamCode())) {
            examReq.setCode(req.getExamName());
        } else {
            examReq.setCode(req.getExamCode());
        }
        if (StringUtils.isBlank(examReq.getCode())) {
            throw new StatusException("OE-ADMIN-1000005", "考试代码不能为空");
        }
        GetExamResp examResp = examCloudService.getExam(examReq);
        ExamBean examBean = examResp.getExamBean();

        CourseBean courseBean = this.queryCourse(req.getRootOrgId(), null, req.getCourseCode());

        ExamRecordDataEntity selectCondition = new ExamRecordDataEntity();
        selectCondition.setExamId(examBean.getId());
        selectCondition.setCourseId(courseBean.getId());
        if (StringUtils.isNotBlank(req.getIdentityNumber())) {
            selectCondition.setIdentityNumber(req.getIdentityNumber());
        }
        if (StringUtils.isNotBlank(req.getStudentCode())) {
            selectCondition.setStudentCode(req.getStudentCode());
        }

        //查询考试记录
        List<ExamRecordDataEntity> examRecordList = examRecordDataRepo.findAll(Example.of(selectCondition));

        List<ScoreDataBean> scoreDataBeanList = new ArrayList<>();
        for (ExamRecordDataEntity examRecordData : examRecordList) {
            ExamScoreEntity examScore = examScoreRepo.findByExamRecordDataId(examRecordData.getId());
            if (examScore == null) {
                throw new StatusException("100001", "该考生未交卷");
            }

            ScoreDataBean scoreDataBean = new ScoreDataBean();
            scoreDataBean.setExamRecordDataId(examRecordData.getId());
            scoreDataBean.setStudentCode(examRecordData.getStudentCode());
            scoreDataBean.setStudentName(examRecordData.getStudentName());
            scoreDataBean.setIdentityNumber(examRecordData.getIdentityNumber());

            scoreDataBean.setCourseCode(courseBean.getCode());
            scoreDataBean.setCourseName(courseBean.getName());
            scoreDataBean.setStartTime(examRecordData.getStartTime());
            scoreDataBean.setEndTime(examRecordData.getEndTime());

            scoreDataBean.setIsWarn(examRecordData.getIsWarn());
            scoreDataBean.setIsAudit(examRecordData.getIsAudit());
            scoreDataBean.setIsIllegality(examRecordData.getIsIllegality());
            scoreDataBean.setTotalScore(examScore.getTotalScore());
            scoreDataBeanList.add(scoreDataBean);
        }

        QueryScoreDataResp queryScoreDataResp = new QueryScoreDataResp();
        queryScoreDataResp.setScoreDataBeanList(scoreDataBeanList);
        return queryScoreDataResp;
    }

    @Override
    @ApiOperation(value = "获取成绩数据列表（分页）")
    @PostMapping("/queryScoreList")
    public QueryScoreListResp queryScoreList(@RequestBody QueryScoreListReq req) {
        if (req.getRootOrgId() == null) {
            throw new StatusException("OE-ADMIN-1001005", "rootOrgId不能为空");
        }

        if (req.getPageNo() == null || req.getPageNo() < 1) {
            req.setPageNo(1);// 第几页，起始为1
        }
        if (req.getPageSize() == null || req.getPageSize() < 1) {
            req.setPageSize(10);// 每页条数，默认值
        }
        if (req.getPageSize() > 100) {
            req.setPageSize(100);// 每页条数，限制最大100条
        }

        final String pattern = "yyyy-MM-dd HH:mm:ss";
        SimpleDateFormat sdf = new SimpleDateFormat(pattern);

        final Date queryStartTime;
        try {
            if (StringUtils.isNotBlank(req.getQueryStartTime())) {
                queryStartTime = sdf.parse(req.getQueryStartTime());
            } else {
                queryStartTime = null;
            }
        } catch (Exception e) {
            throw new StatusException("OE-ADMIN-1001005", "queryStartTime值有误，格式：" + pattern);
        }

        final Date queryEndTime;
        try {
            if (StringUtils.isNotBlank(req.getQueryEndTime())) {
                queryEndTime = sdf.parse(req.getQueryEndTime());
            } else {
                queryEndTime = null;
            }
        } catch (Exception e) {
            throw new StatusException("OE-ADMIN-1001005", "queryEndTime值有误，格式：" + pattern);
        }

        GetExamReq examReq = new GetExamReq();
        examReq.setRootOrgId(req.getRootOrgId());
        if (StringUtils.isEmpty(req.getExamCode())) {
            // 如果考试代码为空，则将考试名称作为考试代码
            examReq.setCode(req.getExamName());
        } else {
            examReq.setCode(req.getExamCode());
        }
        if (StringUtils.isBlank(examReq.getCode())) {
            throw new StatusException("OE-ADMIN-1001005", "考试代码不能为空");
        }
        GetExamResp examResp = examCloudService.getExam(examReq);
        final Long examId = examResp.getExamBean().getId();

        final CourseBean queryCourse;
        if (StringUtils.isNotBlank(req.getCourseCode())) {
            queryCourse = this.queryCourse(req.getRootOrgId(), null, req.getCourseCode());
        } else {
            queryCourse = null;
        }

        Specification<ExamRecordDataEntity> spec = (Specification<ExamRecordDataEntity>) (root, query, builder) -> {
            List<Predicate> predicates = new ArrayList<>();
            predicates.add(builder.equal(root.get("examId"), examId));

            if (queryCourse != null) {
                predicates.add(builder.equal(root.get("courseId"), queryCourse.getId()));
            }

            predicates.add(builder.notEqual(root.get("examRecordStatus"), ExamRecordStatus.EXAM_INVALID));

            if (queryStartTime != null) {
                predicates.add(builder.greaterThanOrEqualTo(root.get("updateTime"), queryStartTime));
            }

            if (queryEndTime != null) {
                predicates.add(builder.lessThanOrEqualTo(root.get("updateTime"), queryEndTime));
            }

            return builder.and(predicates.toArray(new Predicate[predicates.size()]));
        };

        Pageable pageable = PageRequest.of(req.getPageNo() - 1, req.getPageSize());
        Page<ExamRecordDataEntity> page = examRecordDataRepo.findAll(spec, pageable);
        List<ExamRecordDataEntity> examRecordDataEntities = page.getContent();

        QueryScoreListResp queryResp = new QueryScoreListResp();
        queryResp.setTotalElements(page.getTotalElements());
        queryResp.setTotalPages(page.getTotalPages());
        List<ScoreDataBean> scoreList = new ArrayList<>();
        queryResp.setScoreList(scoreList);

        if (CollectionUtils.isEmpty(examRecordDataEntities)) {
            return queryResp;
        }

        List<Long> examRecordDataIds = examRecordDataEntities.stream().map(ExamRecordDataEntity::getId).collect(Collectors.toList());
        List<ExamScoreEntity> examScoreEntities = examScoreRepo.findByExamRecordDataIdIn(examRecordDataIds);
        if (CollectionUtils.isEmpty(examScoreEntities)) {
            LOGGER.info("examScoreEntities size is empty. ");
            return queryResp;
        }

        LOGGER.info("examRecordData size is " + examRecordDataEntities.size() + ", examScore size is " + examScoreEntities.size());
        Map<Long, ExamScoreEntity> examScoreMaps = examScoreEntities.stream().collect(Collectors.toMap(ExamScoreEntity::getExamRecordDataId, v -> v, (k, v) -> v));

        Map<Long, CourseBean> courseMaps = new HashMap<>();
        for (ExamRecordDataEntity examRecordData : examRecordDataEntities) {
            ExamScoreEntity examScore = examScoreMaps.get(examRecordData.getId());
            if (examScore == null) {
                LOGGER.debug("examSore not exist. examRecordDataId = " + examRecordData.getId());
                continue;
            }

            ScoreDataBean score = new ScoreDataBean();
            if (queryCourse != null) {
                score.setCourseCode(queryCourse.getCode());
                score.setCourseName(queryCourse.getName());
            } else {
                CourseBean course = courseMaps.get(examRecordData.getCourseId());
                if (course == null) {
                    course = this.queryCourse(examRecordData.getRootOrgId(), examRecordData.getCourseId(), null);
                    courseMaps.put(course.getId(), course);
                }
                score.setCourseCode(course.getCode());
                score.setCourseName(course.getName());
            }
            score.setStartTime(examRecordData.getStartTime());
            score.setEndTime(examRecordData.getEndTime());
            score.setExamRecordDataId(examRecordData.getId());
            score.setStudentCode(examRecordData.getStudentCode());
            score.setStudentName(examRecordData.getStudentName());
            score.setIdentityNumber(examRecordData.getIdentityNumber());
            score.setIsIllegality(examRecordData.getIsIllegality());
            score.setIsWarn(examRecordData.getIsWarn());
            score.setIsAudit(examRecordData.getIsAudit());
            score.setTotalScore(examScore.getTotalScore());
            scoreList.add(score);
        }

        return queryResp;
    }

    private CourseBean queryCourse(Long rootOrgId, Long courseId, String courseCode) {
        GetCourseReq courseReq = new GetCourseReq();
        courseReq.setRootOrgId(rootOrgId);
        courseReq.setId(courseId);
        courseReq.setCode(courseCode);
        GetCourseResp courseResp = courseCloudService.getCourse(courseReq);
        return courseResp.getCourseBean();
    }

    @ApiOperation(value = "按考试信息，rootOrgId,courseCode，身份证号或学号查询最终成绩信息")
    @PostMapping("/getFinalScoreData")
    @Override
    public GetFinalScoreDataResp getFinalScoreData(@RequestBody GetFinalScoreDataReq req) {
        if (req.getRootOrgId() == null) {
            throw new StatusException("100002", "学校id不允许为空");
        }
        if (req.getExamId() == null) {
            throw new StatusException("100003", "考试id不允许为空");
        }
        //只支持离线和在线考试
        ExamSettingsCacheBean examSettings = ExamCacheTransferHelper.getDefaultCachedExam(req.getExamId());
        if (!(ExamType.ONLINE.toString().equals(examSettings.getExamType()) ||
                ExamType.OFFLINE.toString().equals(examSettings.getExamType()) ||
                ExamType.ONLINE_HOMEWORK.toString().equals(examSettings.getExamType()))) {
            throw new StatusException("100009", "不支持的考试类型");
        }

        if (StringUtils.isEmpty(req.getIdentityNumber()) && StringUtils.isEmpty(req.getStudentCode())) {
            throw new StatusException("100004", "学号和身份证号不能同时为空");
        }
        if (StringUtils.isEmpty(req.getCourseCode())) {
            throw new StatusException("100005", "课程代码不允许为空");
        }
        String identityNumber = req.getIdentityNumber();
        if (StringUtils.isEmpty(identityNumber)) {
            identityNumber = gainBaseDataService.getStudentBean(req.getStudentCode(),
                    req.getRootOrgId()).getIdentityNumber();
            if (StringUtils.isEmpty(identityNumber)) {
                throw new StatusException("100006", "找不到学号对应的学生身份证信息");
            }
        }

        CourseBean courseBean = gainBaseDataService.getCourseBean(req.getCourseCode(), req.getRootOrgId());
        Long courseId = courseBean.getId();
        if (courseId == null) {
            throw new StatusException("100007", "课程代码不正确");
        }

        GetFinalScoreDataResp resp = new GetFinalScoreDataResp();
        //如果该学生缺考，直接返回
        ExamStudentEntity examStudent = examStudentRepo.findByIdentityNumberAndExamIdAndCourseId(
                identityNumber, req.getExamId(), courseId);
        if (examStudent == null) {
            throw new StatusException("100008", "考生不存在");
        }
        Boolean isAbsent = examStudent.getFinished() == null || !examStudent.getFinished();
        if (isAbsent) {
            resp.setAbsent(true);
            return resp;
        }

        //最终生效的考试分数对象
        ExamStudentFinalScoreEntity finalExamScore =
                examStudentFinalScoreService.getFinalEffectiveExamScore(examStudent.getExamStudentId());

        //如果查到最终有分数数据，则直接返回
        if (null != finalExamScore) {
            ScoreDataBean scoreDataBean = buildScoreDataBean(identityNumber, courseBean,
                    finalExamScore.getTotalScore(), finalExamScore.getExamRecordDataId());
            resp.setScoreDataBean(scoreDataBean);
            resp.setAbsent(false);
            resp.setIllegality(false);
            resp.setAudit(scoreDataBean.getIsAudit());
            return resp;
        }

        //如果查不到有效的分数集合，则将所有的考试分数集合均返回
        resp.setAbsent(false);
        List<ExamScoreEntity> allExamScoreList =
                examScoreService.getAllExamScoreList(req.getExamId(), identityNumber, courseId);
        if (allExamScoreList == null) {
            return resp;
        }

        List<ScoreDataBean> scoreDataBeanList = new ArrayList<>();
        for (ExamScoreEntity scoreEntity : allExamScoreList) {
            scoreDataBeanList.add(buildScoreDataBean(identityNumber,
                    courseBean, scoreEntity.getTotalScore(), scoreEntity.getExamRecordDataId()));
        }
        resp.setAllScoreDataBeanList(scoreDataBeanList);
        resp.setAudit(scoreDataBeanList.stream().anyMatch(p -> p.getIsAudit() == null || !p.getIsAudit()));
        resp.setIllegality(scoreDataBeanList.stream().anyMatch(p -> p.getIsIllegality() != null && p.getIsIllegality()));
        return resp;

    }

    private ScoreDataBean buildScoreDataBean(String identityNumber, final CourseBean courseBean,
                                             Double totalScore, Long examRecordDataId) {
        ScoreDataBean scoreDataBean = new ScoreDataBean();
        scoreDataBean.setIdentityNumber(identityNumber);
        scoreDataBean.setCourseCode(courseBean.getCode());
        scoreDataBean.setCourseName(courseBean.getName());

        scoreDataBean.setTotalScore(totalScore);
        ExamRecordDataEntity examRecordData = GlobalHelper.getEntity(examRecordDataRepo,
                examRecordDataId, ExamRecordDataEntity.class);

        scoreDataBean.setEndTime(examRecordData.getEndTime());
        scoreDataBean.setExamId(examRecordData.getExamId());
        scoreDataBean.setExamRecordDataId(examRecordData.getId());
        scoreDataBean.setIsAudit(examRecordData.getIsAudit());
        scoreDataBean.setIsIllegality(examRecordData.getIsIllegality());
        scoreDataBean.setIsWarn(examRecordData.getIsWarn());
        scoreDataBean.setStartTime(examRecordData.getStartTime());
        scoreDataBean.setStudentCode(examRecordData.getStudentCode());
        scoreDataBean.setStudentName(examRecordData.getStudentName());

        ExamSettingsCacheBean examSettings = ExamCacheTransferHelper.getDefaultCachedExam(examRecordData.getExamId());
        scoreDataBean.setExamCode(examSettings.getCode());
        scoreDataBean.setExamName(examSettings.getName());
        return scoreDataBean;
    }

}
