package cn.com.qmth.examcloud.core.oe.admin.dao;

import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Repository;

import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamCaptureQueueEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.enums.ExamCaptureQueueStatus;

import javax.transaction.Transactional;
import java.util.Date;
import java.util.List;

/**
 * @author chenken
 * @date 2018/8/17 14:42
 * @company QMTH
 * @description ExamCaptureQueueRepo
 */
@Repository
public interface ExamCaptureQueueRepo extends JpaRepository<ExamCaptureQueueEntity, Long>, JpaSpecificationExecutor<ExamCaptureQueueEntity> {

    /**
     * @param limit           一次取值数量
     * @param processBatchNum 批次号
     * @return List<ExamCaptureQueueEntity>
     * @description 根据处理批次号查询没有处理过的数据或者是处理失败的数据，按优先级倒序，创建时间升序
     * @author lideyin
     * @date 2019/7/31 16:40
     */
    @Query(value = "select * from ec_oe_exam_capture_queue "
            + " where status IN ('PENDING','PROCESS_FACE_COMPARE_FAILED')  " +
            " and (process_batch_num is null or (process_batch_num is not null and process_batch_num!=?2))"
            + " order by priority desc,id asc limit ?1", nativeQuery = true)
    List<ExamCaptureQueueEntity> findNeedFaceCompareExamCaptureQueuesLimitByProcessBatchNum(Integer limit, String processBatchNum);

    /**
     * 查询人脸比对处理完成，需要百度活体检测的数据，按优先级倒序，创建时间升序
     *
     * @param limit           数量
     * @param processBatchNum 批次号
     * @return List<ExamCaptureQueueEntity>
     */
    @Query(value = "select * from ec_oe_exam_capture_queue "
            + " where status IN ('PROCESS_FACE_COMPARE_COMPLETE','PROCESS_FACELIVENESS_FAILED') and process_batch_num!=?2  "
            + " order by priority desc,id asc limit ?1", nativeQuery = true)
    List<ExamCaptureQueueEntity> findNeedFacelivenessDetectExamCaptureQueuesLimit(Integer limit, String processBatchNum);

    List<ExamCaptureQueueEntity> findByStatusOrderByCreationTimeAsc(ExamCaptureQueueStatus status);

    List<ExamCaptureQueueEntity> findByExamRecordDataId(Long examRecordDataId);

    /**
     * 是否存在未处理的图片抓拍队列(处理完成的队列数据都会清理掉)
     *
     * @param examRecordDataId
     * @return
     */
    @Query(value = "select * from ec_oe_exam_capture_queue where exam_record_data_id=?1 limit 1", nativeQuery = true)
    ExamCaptureQueueEntity existsUnhandledByExamRecordDataId(Long examRecordDataId);

    /**
     * 修改数据状态为处理中
     *
     * @param id 主键ID
     */
    @Transactional
    @Modifying
    @Query(value = "update ec_oe_exam_capture_queue set status = 'PROCESSING' where id = ?1", nativeQuery = true)
    void updateExamCaptureQueueStatusWithProcessing(Long id);


    /**
     * @param priority         优先级(值越大，优先级越高)
     * @param examRecordDataId 考试记录id
     * @description 更新考试抓拍队列优先级
     * @date 2019/7/31 16:46
     * @author lideyin
     */
    @Transactional
    @Modifying
    @Query(value = "update ec_oe_exam_capture_queue set priority=?1 where exam_record_data_id=?2", nativeQuery = true)
    void updateExamCaptureQueuePriority(int priority, Long examRecordDataId);

    /**
     * @param captureQueueId
     * @param errorMsg
     * @param status
     * @param batchNumber
     * @param updateDate
     */
    @Transactional
    @Modifying
    @Query(value = "update ec_oe_exam_capture_queue set error_msg=?2,`status`=?3,process_batch_num=?4,error_num=error_num+1,update_time=?5 where id=?1", nativeQuery = true)
    void saveExamCaptureQueueEntityByFailed(Long captureQueueId, String errorMsg, String status, String batchNumber,
                                            Date updateDate);

    @Transactional
    @Modifying
    @Query(value = "update ec_oe_exam_capture_queue set is_pass=?2,is_stranger=?3,`status`=?4,face_compare_result=?5,face_compare_start_time=?6,update_time=?7 where id=?1", nativeQuery = true)
    void saveExamCaptureQueueEntityBySuccessful(Long captureQueueId, Boolean isPass, Boolean isStranger, String status,
                                                String faceCompareResult,Long faceCompareStartTime,Date updateDate);

    /**
     * 更新批次号
     * @param captureQueueId 队列id
     * @param batchNum 批次号
     */
    @Transactional
    @Modifying
    @Query(value = "update ec_oe_exam_capture_queue set process_batch_num=?2 where id=?1", nativeQuery = true)
    void updateProcessBatchNum(Long captureQueueId, String batchNum);
}
