package cn.com.qmth.examcloud.core.oe.admin.dao;

import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamRecordDataEntity;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * @author chenken
 * @date 2018年8月13日 下午2:02:25
 * @company QMTH
 * @description ExamRecordDataRepo.java
 */
@Repository
public interface ExamRecordDataRepo extends JpaRepository<ExamRecordDataEntity, Long>, JpaSpecificationExecutor<ExamRecordDataEntity> {

    List<ExamRecordDataEntity> findByExamStudentId(Long examStudentId);

    /**
     * 查询有效的考试记录(已结束且未违纪的)
     *
     * @param examStudentIdList
     * @return
     */
    @Query(value = "select erd.* " +
            "from ec_oe_exam_record_data erd " +
            "where erd.exam_record_status in('EXAM_END','EXAM_OVERDUE') " +
            "and erd.is_illegality=0 and (is_warn=0 or is_warn=1  and erd.is_audit=1) and erd.exam_student_id in ?1", nativeQuery = true)
    List<ExamRecordDataEntity> findEffectiveDataByExamStudentIdList(List<Long> examStudentIdList);

    List<ExamRecordDataEntity> findByExamIdAndStudentCode(Long examId, String studentCode);

    List<ExamRecordDataEntity> findByExamIdAndIdentityNumberAndCourseId(Long examId, String identityNumber, Long courseId);

    /**
     * 同步更新考生信息
     *
     * @param studentName
     * @param studentCode
     * @param infoCollector
     */
    @javax.transaction.Transactional
    @Modifying
    @Query(nativeQuery = true, value = "update ec_oe_exam_record_data set student_name = ?2,student_code = ?3,info_collector=?4 where exam_student_id = ?1")
    void syncUpdateExamStudentInfo(Long examStudentId, String studentName, String studentCode, String infoCollector);

    @javax.transaction.Transactional
    @Modifying
    @Query(nativeQuery = true, value = "update ec_oe_exam_record_data set student_name = ?2  where student_id = ?1")
    void updateStudentName(Long studentId, String studentName);

    @javax.transaction.Transactional
    @Modifying
    @Query(nativeQuery = true, value = "update ec_oe_exam_record_data set course_level = ?2 where course_id = ?1")
    void updateCourse(Long courseId, String courseLevel);

    /**
     * 根据studentId查询状态为"考试中"的在线考试记录
     *
     * @param studentId
     * @return
     */
    @Query(value = " select * from ec_oe_exam_record_data t1 " +
            " where t1.student_id = ?1  " +
            " and t1.exam_record_status = 'EXAM_ING' " +
            " and t1.exam_type = 'ONLINE'", nativeQuery = true)
    ExamRecordDataEntity findOnlineExamingRecordByStudentId(Long studentId);

    /**
     * 更新考试记录中的考试作答记录id
     *
     * @param examRecordQuestionId 考试作答记录id
     * @param id                   examRecordDataId
     * @return
     */
    @Transactional
    @Modifying
    @Query(value = "update ec_oe_exam_record_data set exam_record_questions_id=?1 where id=?2", nativeQuery = true)
    int updateExamRecordDataQuestionIdById(String examRecordQuestionId, Long id);

    /**
     * 更新活体检测结果
     *
     * @param faceVerifyResult
     * @param id
     * @return
     */
    @Transactional
    @Modifying
    @Query(value = "update ec_oe_exam_record_data set face_verify_result = ?1  where student_id = ?2", nativeQuery = true)
    int updateFaceVerifyResult(String faceVerifyResult, Long id);

    @Query(value = "select * from ec_oe_exam_record_data " +
            "where exam_record_status in('EXAM_END','EXAM_OVERDUE') and is_illegality=0 and (is_warn=0 or is_warn=1  and is_audit=1) " +
            "and exam_id=?1 and id>=?2 order by id asc limit ?3", nativeQuery = true)
    List<ExamRecordDataEntity> findLimitedDataByExamIdAndIdMoreThan(Long examId, Long startExamRecordId, int rowCount);

    @Query(value = "select record_data.* from ec_oe_exam_record_data record_data " +
            "where record_data.exam_record_status in('EXAM_END','EXAM_OVERDUE') and record_data.exam_id=?1 " +
            "AND record_data.is_audit=0 ", nativeQuery = true)
    List<ExamRecordDataEntity> findDataByExamId(Long examId);
    
    @Query(value = "select t.id from ec_oe_exam_record_data t where t.base_paper_id=?1 limit 1", nativeQuery = true)
    Long getRecordIdByPaperId(String basePaperId);
}
