/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-08-15 14:15:19.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.oe.admin.service;

import cn.com.qmth.examcloud.api.commons.enums.ExamType;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.OnHandExamInfo;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examstudent.*;
import org.springframework.data.domain.Page;

import java.util.List;

/**
 * 考生信息接口
 *
 * @author: QMTH
 * @since: 2018/8/15
 */
public interface ExamStudentService {

    /**
     * 保存考生数据
     *
     * @param examStudents
     * @return
     */
    List<Long> saveExamStudentList(List<ExamStudentInfo> examStudents);

    /**
     * 同步考生(部分)数据
     *
     * @param examStudents
     * @return
     */
    void syncExamStudentPartData(List<ExamStudentPartInfo> examStudents);

    /**
     * 同步清理考生数据
     *
     * @param examId
     * @return
     */
    void syncRemoveExamStudentByExamId(Long examId);

    /**
     * 查询考生列表(分页)
     *
     * @param query
     * @return
     */
    Page<ExamStudentInfo> getExamStudentListPage(ExamStudentQuery query);


    public List<ExamStudentInfo> getExamStudentInfoList(ExamStudentQuery query);

    /**
     * 查询重考考生列表(分页)
     *
     * @param query
     * @return
     */
    Page<ExamStudentInfo> getReExamineStudentList(ExamStudentQuery query);

    /**
     * 考试概况-按考试完成进度统计
     *
     * @param examId
     * @return
     */
    ExamStudentFinishedStatistic getExamStudentStatisticByFinished(Long examId, Long examStageId);

    /**
     * 考试概况-按学习中心统计
     *
     * @param examId
     * @param orgId
     * @param examStageId
     * @return
     */
    List<ExamStudentOrgStatistic> getExamStudentStatisticByOrg(Long examId, Long examStageId, Long orgId);

    /**
     * 获取考生信息
     *
     * @param examStudentId
     * @return
     */
    ExamStudentInfo getExamStudentInfo(Long examStudentId);

    /**
     * 判断考生是否启用
     *
     * @param examStudentId
     * @return
     */
    boolean isEnableExamStudent(Long examStudentId);

    /**
     * 查询课程信息
     *
     * @param examId
     * @param orgId
     * @return
     */
    List<Long> findCoursesFromExamStudent(Long examId, Long examStageId, Long orgId);

    /**
     * 设置重考
     *
     * @param examStudentId
     * @param reexamineType
     * @param reexamineDetail
     */
    public void setReexamine(Long examStudentId, String reexamineType, String reexamineDetail);

    /**
     * 课程进度查询
     *
     * @param examId
     * @param courseId
     */
    public List<CourseProgressInfo> queryCourseProgressInfos(Long examId, Long examStageId, Long courseId, String orderColumn);

    /**
     * @return java.util.List<cn.com.qmth.examcloud.core.oe.admin.service.bean.examstudent.ExamStudentInfo>
     * @Author lideyin
     * @Description 获取指定数量的考生数据
     * @Date 2019/7/18 14:36
     * @Param [ examId, startId, size]
     */
    public List<ExamStudentInfo> getLimitExamStudentList(Long examId, Long startId, Integer size);

    /**
     * 获取在线考试待考列表
     *
     * @param studentId 学生id
     * @param examType  考试类型
     * @return
     */
    public List<OnHandExamInfo> queryOnlineExamList(Long studentId, ExamType examType);

    List<ExamStudentInfo> getExamStudentInfoListForAsync(ExamStudentQuery query);

    List<OnHandExamInfo> queryOnlineExamEndList(Long userId, ExamType online);

}
