package cn.com.qmth.examcloud.core.oe.admin.service.impl;

import cn.com.qmth.examcloud.commons.util.HttpMethod;
import cn.com.qmth.examcloud.commons.util.OKHttpUtil;
import cn.com.qmth.examcloud.core.oe.admin.base.Constants;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamRecordDataRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamScoreObtainQueueRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamScoreObtainQueueEntity;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamScoreObtainQueueService;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.NotifyUrlInfo;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.support.cache.bean.SysPropertyCacheBean;
import cn.com.qmth.examcloud.support.enums.ExamProperties;
import cn.com.qmth.examcloud.support.helper.ExamCacheTransferHelper;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.validation.constraints.NotNull;
import java.util.Date;

/**
 * @author chenken
 * @date 2018年11月27日 下午2:05:32
 * @company QMTH
 * @description 成绩获取队列
 */
@Service("examScoreObtainQueueService")
public class ExamScoreObtainQueueServiceImpl implements ExamScoreObtainQueueService {

    private static final Logger log = LoggerFactory.getLogger(ExamScoreObtainQueueServiceImpl.class);

    @Autowired
    private ExamScoreObtainQueueRepo examScoreObtainQueueRepo;

    @Autowired
    private ExamRecordDataRepo examRecordDataRepo;

    @Override
    public void saveExamScoreObtainQueue(Long examRecordDataId) {

        ExamRecordDataEntity examRecordData =
                GlobalHelper.getEntity(examRecordDataRepo, examRecordDataId, ExamRecordDataEntity.class);
        //得到是否推送分数属性值
        String isPushScore = ExamCacheTransferHelper.getCachedExamProperty(examRecordData.getExamId(),
                examRecordData.getStudentId(), ExamProperties.PUSH_SCORE.name()).getValue();
        if (Constants.isTrue.equals(isPushScore)) {
            NotifyUrlInfo notifyUrlInfo = getNotifyUrlInfo(examRecordData.getRootOrgId());
            if (StringUtils.isNotBlank(notifyUrlInfo.getNotifyUrl())) {
                //保存队列
                ExamScoreObtainQueueEntity examScoreObtainQueue = examScoreObtainQueueRepo.findByExamRecordDataId(examRecordData.getId());
                if (examScoreObtainQueue == null) {
                    examScoreObtainQueue = new ExamScoreObtainQueueEntity();
                    examScoreObtainQueue.setCreationTime(new Date());
                }
                examScoreObtainQueue.setExamId(examRecordData.getExamId());
                examScoreObtainQueue.setExamRecordDataId(examRecordData.getId());
                examScoreObtainQueue.setRootOrgId(examRecordData.getRootOrgId());
                examScoreObtainQueue.setIsValid(true);
                examScoreObtainQueue.setUpdateTime(new Date());
                examScoreObtainQueueRepo.save(examScoreObtainQueue);
            }
        }
    }

    @Override
    public void sendObtainScoreNotify(Long rootOrgId) {
        if (rootOrgId == null) {
            return;
        }
        NotifyUrlInfo notifyUrlInfo = getNotifyUrlInfo(rootOrgId);

        if (StringUtils.isNotBlank(notifyUrlInfo.getNotifyUrl())) {
            log.info("ExamScoreObtainQueueServiceImpl-notifyUrl:" + notifyUrlInfo.getNotifyUrl());
            try {
                OKHttpUtil.call(notifyUrlInfo.getHttpMethod(), notifyUrlInfo.getNotifyUrl());
            } catch (Exception e) {
                log.error("ExamScoreObtainQueueServiceImpl-sendNotifyError3，" + e.getMessage(), e);

            }
        }
    }

    /**
     * 根据组织机构获取通知路径相关信息
     *
     * @param rootOrgId 组织机构id
     * @return NotifyUrlInfo
     */
    @NotNull
    @Override
    public NotifyUrlInfo getNotifyUrlInfo(Long rootOrgId) {
        NotifyUrlInfo notifyUrlInfo = new NotifyUrlInfo();
        if (rootOrgId == null) {
            return notifyUrlInfo;
        }
        String notifyUrlKey = Constants.OE_EXAM_SCORE_NOTIFY_URL_PREFIX + rootOrgId;
        String notifyUrlMethodKey = Constants.OE_EXAM_SCORE_NOTIFY_URL_HTTP_METHOD_PREFIX + rootOrgId;
        SysPropertyCacheBean notifyUrlKeyProperty = CacheHelper.getSysProperty(notifyUrlKey);
        SysPropertyCacheBean notifyUrlMethodKeyProperty = CacheHelper.getSysProperty(notifyUrlMethodKey);
        //默认http调用方式：post
        HttpMethod defaultHttpMethod = HttpMethod.POST;
        if (!notifyUrlKeyProperty.getHasValue() || !notifyUrlKeyProperty.getHasValue()) {
            log.error("ExamScoreObtainQueueServiceImpl-sendNotifyError001，未找到组织机构id为：" + rootOrgId + "的配置路径");
        } else {
            notifyUrlInfo.setNotifyUrl(notifyUrlKeyProperty.getValue().toString());
        }
        if (null != notifyUrlMethodKeyProperty && null != notifyUrlMethodKeyProperty.getValue()) {
            try {
                notifyUrlInfo.setHttpMethod(HttpMethod.valueOf(notifyUrlMethodKeyProperty.getValue().toString()));
            } catch (Exception e) {
                log.error("ExamScoreObtainQueueServiceImpl-sendNotifyError002，notifyUrlMethodKey：" + notifyUrlMethodKey + "格式不正确");
            }
        } else {
            notifyUrlInfo.setHttpMethod(defaultHttpMethod);
        }
        return notifyUrlInfo;

    }


}
